"""
NewFunctionsPYC
~~~~~~~~~~

This module is for adding new functions to pycord
To make it easier for new people to get started on pycord

:copyright: (c) 2023-present Marciel404
:license: MIT, see LICENSE for more details.

:pycord-site: https://pycord.dev/ (English)
:pycord-discord-server: https://discord.gg/pycord (English)

:my-server: https://discord.gg/2UtE8tMyh5 (Portuguese)
"""

__name__ = "NewFunctionsPYC"
__version__ = "1.0.1"
__author__ = "Marciel404"
__license__ = "MIT"

from .embeds.embed import *
from .client.loader import *
from .commands.prefix import *
from .commands.slash import *
from .commands.hybrid import *




import discord
from discord.ext import command

slashContext = discord.Interaction

def Slash_Command(name: str = None,
                description: str = None,
                guild_only: bool = False,
                nsfw: bool = False,
                options: list = None,
                guild_ids: list = None,
                name_localizations: dict = None,
                description_localizations: dict = None,
                **kwargs
                ) -> any:
        
    if options == None:
        options = []
    if name_localizations == None:
        name_localizations = {}
    if description_localizations == None:
        description_localizations = {}
    if name != None:
        name = name.lower().replace(" ","_")
    
    return discord.slash_command(
        name = name, 
        description = description,
        guild_only = guild_only,
        nsfw = nsfw,
        options = options,
        name_localizations = name_localizations,
        description_localizations = description_localizations,
        guild_ids = guild_ids,
        **kwargs
    )


from os import listdir
import discord
from discord.ext import commands, bridge
from discord import Intents, Game
from ..commands.hybrid import hybridCtx

defaultIntents: Intents = Intents.default()
defaultIntents.message_content = True

class client(bridge.Bot):

    def __init__(self,
            token: str,
            command_prefix: str = None,
            Intents: Intents = None,
            poweredby: bool = True,
            case_insensitive: bool = True,
            auto_sync_commands: bool = True,
            **options
        ):

        if token == None or token == "":
            raise print("Token is a argument required")
        if command_prefix == None:
            command_prefix = "!"
        if Intents == None:
            Intents = defaultIntents
        
        self.token: str = token
        self.poweredby: bool = poweredby

        super().__init__(
            command_prefix = command_prefix,  
            intents = Intents,
            auto_sync_commands = auto_sync_commands,
            case_insensitive = case_insensitive,
            **options
        )
    
    async def on_ready(self):
        if self.poweredby == True:
            await self.change_presence(activity=Game(name="PoweredBy PyCord and NewFunctionsPYC"))
            print("PoweredBy PyCord and NewFunctionsPYC")
    
    def load_cogs(self, folder: str):

        if folder.startswith("./") or folder.startswith("."):
            folder = folder.strip("./")
            
        for cogs in listdir(f"{folder}"):
            if cogs.endswith(".py") and not cogs.startswith("__"):
                self.load_extensions("{0}.{1}".format(folder.replace("/", "."),cogs[:-3]))

    def __run__(self):
        if self.token == None or self.token == "":
            raise print("Token is a argument required")
        try:
            self.run(self.token)
        except Exception as error:
            return print(error)

class Client(client):
    """Represents a discord bot.

    This class is a subclass of :class:`NewFunctionsPYC.client` and as a result
    anything that you can do with a :class:`discord.Bot` you can do with
    this bot.
    """

class Bot(client):
    """Represents a discord bot.

    This class is a subclass of :class:`NewFunctionsPYC.client` and as a result
    anything that you can do with a :class:`discord.Bot` you can do with
    this bot.
    """

class bot(client):
    """Represents a discord bot.

    This class is a subclass of :class:`NewFunctionsPYC.client` and as a result
    anything that you can do with a :class:`discord.Bot` you can do with
    this bot.
    """


import discord
from discord.ext.bridge import bridge_command, BridgeContext

hybridContext = BridgeContext

def HybridCommand(name: str,
                description: str = None,
                guild_only: bool = False,
                nsfw: bool = False,
                options: list = None,
                guild_ids: list = None,
                name_localizations: dict = None,
                description_localizations: dict = None,
                usage: str = None, 
                aliases: list = None,
                enable: bool = True,
                help: str = None,
                **kwargs) -> any:
    
    if options == None:
        options = []
    if name_localizations == None:
        name_localizations = {}
    if description_localizations == None:
        description_localizations = {}
    if name != None:
        name = name.lower().replace(" ","_")
    if description == None:
        description = "No description provided"
    if usage == None:
        usage = "Not provided"
    if description == None:
        description = "Not provided"
    if help == None:
        help = "Not provided"
    if aliases == None:
        aliases = []

    return bridge_command(
        name = name,
        description = description,
        name_localizations = name_localizations,
        description_localizations = description_localizations,
        guild_only = guild_only,
        nsfw = nsfw,
        guild_ids = guild_ids,
        usage = usage, 
        aliases = aliases,
        enable = enable,
        help = help,
        kwargs=kwargs
    )


import discord
from discord.ext import commands

prefixContext = commands.Context

def CommandPrefix(name: str = None, 
                usage: str = None, 
                description: str = None,
                aliases: list = None,
                enable: bool = True,
                help: str = None,
                **kwargs
                )-> any:

    if usage == None:
        usage = "Not provided"
    if description == None:
        description = "Not provided"
    if help == None:
        help = "Not provided"
    if aliases == None:
        aliases = []
    if name != None:
        name = name.lower().replace(" ","_")

    return commands.command(
        name = name,
        usage = usage,
        description = description,
        aliases = aliases,
        help = help,
        enable = enable,
        **kwargs
    )


import discord

class EmbedBuilder:

    def __init__(self) -> None:

        self.settitle: str = ""
        self.setdescription: str = ""
        self.setcolour: int = discord.Colour.dark_theme()
        self.addfields: list = []
        self.insertfield: list = []
        self.setfooter: dict = {}
        self.setauthor: dict = {}
        self.setimage: str = None
        self.setthumb: str = None
        self.removefield: int = None
        self.removeauthor: bool = False
        self.removeimage: bool = False
        self.removethumb: bool = False
        self.removefooter: bool = False
        self.clearfields: bool = False

    def set_title(self, title: str) -> str:
        self.settitle = str(title)

    def set_description(self, description: str) -> str:
        self.setdescription = str(description)

    def set_color(self, color: int) -> int:
        self.setcolour = int(color)

    def set_colour(self, color: int) -> int:
        self.setcolour = int(color)
    
    def add_field(self, name: str, value: str, inline: bool = True) -> list:
        
        self.addfields.append(
            {
                "name": name, 
                "value": value, 
                "inline": inline
            }
        )
    
    def insert_field_at(self, index: int, name: str, value: str, inline: bool = True):

        self.insertfield.append(
            {
                "index": index,
                "name": name,
                "value": value,
                "inline": inline
            }
        )

    
    def set_footer(self, text: str, icon_url: str = "") -> dict:

        self.setfooter = {
            "text": text,
            "icon_url": icon_url
        }
    
    def set_author(self, name: str, url: str = "", icon_url: str = "") -> dict:

        self.setauthor = {
            "name": name,
            "url": url,
            "icon_url": icon_url
        }
    
    def set_image(self, url: str) -> str:
        self.setimage = str(url)
    
    def set_thumbinail(self, url: str) -> str:
        self.setthumb = str(url)

    def remove_title(self) -> str:
        self.title = ""
    
    def remove_description(self) -> str:
        self.description = ""

    def remove_color(self) -> int:
        self.setcolour = discord.Color.dark_theme()
    
    def remove_colour(self) -> int:
        self.setcolour = discord.Color.dark_theme()
    
    def remove_author(self) -> bool:
        self.removeauthor = True
    
    def remove_image(self) -> bool:
        self.removeimage = True
    
    def remove_thumbnail(self) -> bool:
        self.removethumb = True
    
    def remove_field(self, index: int) -> int:
        self.removefield = int(index)
    
    def remove_footer(self) -> bool:
        self.removefooter = True
    
    def clear_fields(self) -> bool:
        self.clearfiels = True
    
    def build(self) -> discord.Embed:

        e = discord.Embed(
            title = self.settitle,
            description = self.setdescription,
            color = self.setcolour
        )

        if self.setauthor != {}: 
            e.set_author(
                name = self.setauthor["name"], 
                url = self.setauthor["url"], 
                icon_url = self.setauthor["icon_url"]
            )

        if self.setfooter != {}:
            e.set_footer(
                text = self.setfooter["text"],
                icon_url = self.setfooter["icon_url"]
            )

        if self.add_fields != []:
            for x in self.addfields: 
                e.add_field(
                    name = x["name"],
                    value = x["value"],
                    inline = x["inline"]
                )
        
        if self.insert_field_at != []:
            for x in self.insertfield:
                e.insert_field_at(
                    index = x["index"],
                    name = x["name"],
                    value = x["value"],
                    inline = x["inline"]
                )

        if self.setthumb != None: 
            e.set_thumbnail(self.thumb)

        if self.setimage != None: 
            e.set_image(url = self.url)

        if self.removefield != None: 
            e.remove_field(self.remove_field)

        if self.removefooter != False: 
            e.remove_footer()

        if self.removeauthor != False: 
            e.remove_author()

        if self.removeimage != False: 
            e.remove_image()

        if self.removethumb != False: 
            e.remove_thumbnail()
        
        if self.clearfiels != False:
            e.clear_fields()
        
        return e
